@@ -217,6 +217,12 @@ void _embind_register_class_class_property(
     const char* setterSignature,
     GenericFunction setter);
 
+void _embind_register_iterable(
+    TYPEID classType,
+    TYPEID elementType,
+    const char* sizeMethodName,
+    const char* getMethodName);
+
 EM_VAL _embind_create_inheriting_constructor(
     const char* constructorName,
     TYPEID wrapperType,
@@ -1418,6 +1424,32 @@ struct RegisterClassMethod<FunctionTag<Callable, ReturnType (ThisType, Args...)>
     }
 };
 
+// Helper structs for shifting argument indices when a policy is applied.
+template<typename Slot>
+struct ShiftSlot {
+    using type = Slot;
+};
+
+template<int Index>
+struct ShiftSlot<arg<Index>> {
+    using type = arg<Index + 1>;
+};
+
+template<typename Policy>
+struct ShiftPolicy {
+    using type = Policy;
+};
+
+template<typename Slot>
+struct ShiftPolicy<allow_raw_pointer<Slot>> {
+    using type = allow_raw_pointer<typename ShiftSlot<Slot>::type>;
+};
+
+template<typename Slot>
+struct ShiftPolicy<nonnull<Slot>> {
+    using type = nonnull<typename ShiftSlot<Slot>::type>;
+};
+
 } // end namespace internal
 
 template<typename... ConstructorArgs>
@@ -1526,10 +1558,11 @@ public:
         return *this;
     }
 
-    template<typename WrapperType, typename... ConstructorArgs>
+    template<typename WrapperType, typename... ConstructorArgs, typename... Policies>
     EMSCRIPTEN_ALWAYS_INLINE const class_& allow_subclass(
         const char* wrapperClassName,
-        ::emscripten::constructor<ConstructorArgs...> = ::emscripten::constructor<>()
+        ::emscripten::constructor<ConstructorArgs...> = ::emscripten::constructor<>(),
+        Policies... policies
     ) const {
         using namespace internal;
 
@@ -1543,18 +1576,20 @@ public:
             class_function(
                 "implement",
                 &wrapped_new<WrapperType*, WrapperType, val, ConstructorArgs...>,
-                allow_raw_pointer<ret_val>(), nonnull<ret_val>())
+                allow_raw_pointer<ret_val>(), nonnull<ret_val>(),
+                typename ShiftPolicy<Policies>::type()...)
             .class_function(
                 "extend",
                 &wrapped_extend<WrapperType>)
             ;
     }
 
-    template<typename WrapperType, typename PointerType, typename... ConstructorArgs>
+    template<typename WrapperType, typename PointerType, typename... ConstructorArgs, typename... Policies>
     EMSCRIPTEN_ALWAYS_INLINE const class_& allow_subclass(
         const char* wrapperClassName,
         const char* pointerName,
-        ::emscripten::constructor<ConstructorArgs...> = ::emscripten::constructor<>()
+        ::emscripten::constructor<ConstructorArgs...> = ::emscripten::constructor<>(),
+        Policies... policies
     ) const {
         using namespace internal;
 
@@ -1569,7 +1604,8 @@ public:
             class_function(
                 "implement",
                 &wrapped_new<PointerType, WrapperType, val, ConstructorArgs...>,
-                allow_raw_pointer<ret_val>())
+                allow_raw_pointer<ret_val>(),
+                typename ShiftPolicy<Policies>::type()...)
             .class_function(
                 "extend",
                 &wrapped_extend<WrapperType>)
@@ -1587,6 +1623,19 @@ public:
         return *this;
     }
 
+    template<typename ElementType>
+    EMSCRIPTEN_ALWAYS_INLINE const class_& iterable(
+        const char* sizeMethodName,
+        const char* getMethodName) const {
+        using namespace internal;
+        _embind_register_iterable(
+            TypeID<ClassType>::get(),
+            TypeID<ElementType>::get(),
+            sizeMethodName,
+            getMethodName);
+        return *this;
+    }
+
     template<
         typename FieldType,
         typename... Policies,
@@ -1847,6 +1896,8 @@ template<typename T, class Allocator=std::allocator<T>>
 class_<std::vector<T, Allocator>> register_vector(const char* name) {
     typedef std::vector<T, Allocator> VecType;
     register_optional<T>();
+    using VectorElementType =
+        typename internal::RawPointerTransformer<T, std::is_pointer<T>::value>::type;
 
     return class_<VecType>(name)
         .template constructor<>()
@@ -1855,7 +1906,7 @@ class_<std::vector<T, Allocator>> register_vector(const char* name) {
         .function("size", internal::VectorAccess<VecType>::size, allow_raw_pointers())
         .function("get", internal::VectorAccess<VecType>::get, allow_raw_pointers())
         .function("set", internal::VectorAccess<VecType>::set, allow_raw_pointers())
-        ;
+        .template iterable<VectorElementType>("size", "get");
 }
 
 ////////////////////////////////////////////////////////////////////////////////
