@@ -1,5 +1,6 @@
 #pragma once
 
+#include <stdbool.h>
 #include <stddef.h>
 #include <stdint.h>
 #include <emscripten/atomic.h>
@@ -115,7 +116,7 @@ int emscripten_navigator_hardware_concurrency(void);
 // accesses, behavior differs across browsers, see
 //  - https://bugzil.la/1246139
 //  - https://bugs.chromium.org/p/chromium/issues/detail?id=1167449
-int emscripten_atomics_is_lock_free(int byteWidth);
+bool emscripten_atomics_is_lock_free(int byteWidth);
 
 #define emscripten_lock_t volatile uint32_t
 
@@ -189,7 +190,7 @@ void emscripten_lock_busyspin_waitinf_acquire(emscripten_lock_t *lock __attribut
 // NOTE 2: This function will always acquire the lock asynchronously. That is,
 //         the lock will only be attempted to acquire after current control flow
 //         yields back to the browser, so that the Wasm call stack is empty.
-//         This is to guarantee an uniform control flow. If you use this API in
+//         This is to guarantee a uniform control flow. If you use this API in
 //         a Worker, you cannot utilise an infinite loop programming model.
 void emscripten_lock_async_acquire(emscripten_lock_t *lock __attribute__((nonnull)),
                                    emscripten_async_wait_volatile_callback_t asyncWaitFinished __attribute__((nonnull)),
