@@ -5,17 +5,15 @@
  * found in the LICENSE file.
  */
 
-#include <emscripten/emscripten.h>
-#include <emscripten/html5.h>
+#include <emscripten/html5.h> // For `emscripten_date_now`
 #include <time.h>
-#include <stdbool.h>
 #include <sys/time.h>
-#include <threads.h>
-#include "libc.h"
 
-#include "emscripten_internal.h"
+#include "libc.h" // For `weak`
 
-weak time_t __time(time_t *t) {
+// emscripten-specific implementations. These works out slightly smaller than
+// musl's since it they avoid depending on the more general clock_gettime.
+weak time_t time(time_t *t) {
   double ret = emscripten_date_now() / 1000;
   if (t) {
     *t = ret;
@@ -23,21 +21,16 @@ weak time_t __time(time_t *t) {
   return ret;
 }
 
-static thread_local bool checked_monotonic = false;
-static thread_local bool is_monotonic = 0;
-
-weak int __gettimeofday(struct timeval *restrict tv, void *restrict tz) {
+weak int gettimeofday(struct timeval *restrict tv, void *restrict tz) {
   double now_ms = emscripten_date_now();
   long long now_s = now_ms / 1000;
   tv->tv_sec = now_s; // seconds
-  tv->tv_usec = (now_ms - (now_s * 1000)) * 1000; // nicroseconds
+  tv->tv_usec = (now_ms - (now_s * 1000)) * 1000; // microseconds
   return 0;
 }
 
+// Non-standard glibc/BSD extension that is not implemented by musl
 weak int dysize(int year) {
   int leap = ((year % 4 == 0) && ((year % 100 != 0) || (year % 400 == 0)));
   return leap ? 366 : 365;
 }
-
-weak_alias(__time, time);
-weak_alias(__gettimeofday, gettimeofday);
