@@ -17,14 +17,24 @@
 #include <emscripten/threading.h>
 #include <emscripten/emscripten.h>
 
-int emscripten_has_threading_support() { return 0; }
+bool emscripten_has_threading_support() { return false; }
 
 int emscripten_num_logical_cores() { return 1; }
 
-int emscripten_futex_wait(
-  volatile void /*uint32_t*/* addr, uint32_t val, double maxWaitMilliseconds) {
-  // nop
-  return 0; // success
+#ifndef __EMSCRIPTEN_WASM_WORKERS__
+// These low level primites are defined in both pthreads and wasm workers
+// builds.
+
+int emscripten_futex_wait(volatile void /*uint32_t*/* addr,
+                          uint32_t val,
+                          double maxWaitMilliseconds) {
+  if (!addr) {
+    return -EINVAL;
+  }
+  if (*(uint32_t*)addr != val) {
+    return -EWOULDBLOCK;
+  }
+  return -ENOTSUP;
 }
 
 int emscripten_futex_wake(volatile void /*uint32_t*/* addr, int count) {
@@ -32,7 +42,20 @@ int emscripten_futex_wake(volatile void /*uint32_t*/* addr, int count) {
   return 0; // success
 }
 
-int emscripten_is_main_runtime_thread() { return 1; }
+bool emscripten_is_main_runtime_thread() {
+  return true;
+}
+
+void __wait(volatile int *addr, volatile int *waiters, int val, int priv) {}
+
+void __lock(void* ptr) {}
+
+void __unlock(void* ptr) {}
+
+void __acquire_ptc() {}
+
+void __release_ptc() {}
+#endif
 
 void emscripten_main_thread_process_queued_calls() {
   // nop
@@ -376,16 +399,6 @@ int sem_destroy(sem_t *sem) {
   return 0;
 }
 
-void __wait(volatile int *addr, volatile int *waiters, int val, int priv) {}
-
-void __lock(void* ptr) {}
-
-void __unlock(void* ptr) {}
-
-void __acquire_ptc() {}
-
-void __release_ptc() {}
-
 // When pthreads is not enabled, we can't use the Atomics futex api to do
 // proper sleeps, so simulate a busy spin wait loop instead.
 void emscripten_thread_sleep(double msecs) {
