@@ -146,7 +146,7 @@ void emscripten_proxy_execute_queue(em_proxying_queue* q) {
   }
 }
 
-static int do_proxy(em_proxying_queue* q, pthread_t target_thread, task t) {
+static bool do_proxy(em_proxying_queue* q, pthread_t target_thread, task t) {
   assert(q != NULL);
   pthread_mutex_lock(&q->mutex);
   bool is_system_queue = q == &system_proxying_queue;
@@ -159,13 +159,13 @@ static int do_proxy(em_proxying_queue* q, pthread_t target_thread, task t) {
   }
   pthread_mutex_unlock(&q->mutex);
   if (tasks == NULL) {
-    return 0;
+    return false;
   }
 
   return em_task_queue_send(tasks, t);
 }
 
-int emscripten_proxy_async(em_proxying_queue* q,
+bool emscripten_proxy_async(em_proxying_queue* q,
                            pthread_t target_thread,
                            void (*func)(void*),
                            void* arg) {
@@ -384,7 +384,7 @@ static void call_with_ctx(void* arg) {
   ctx->func(ctx, ctx->arg);
 }
 
-int emscripten_proxy_sync_with_ctx(em_proxying_queue* q,
+bool emscripten_proxy_sync_with_ctx(em_proxying_queue* q,
                                    pthread_t target_thread,
                                    void (*func)(em_proxying_ctx*, void*),
                                    void* arg) {
@@ -394,7 +394,7 @@ int emscripten_proxy_sync_with_ctx(em_proxying_queue* q,
   em_proxying_ctx_init_sync(&ctx, func, arg);
   if (!do_proxy(q, target_thread, (task){call_with_ctx, cancel_ctx, &ctx})) {
     em_proxying_ctx_deinit(&ctx);
-    return 0;
+    return false;
   }
   pthread_mutex_lock(&ctx.sync.mutex);
   while (ctx.sync.state == PENDING) {
@@ -413,7 +413,7 @@ static void call_then_finish_task(em_proxying_ctx* ctx, void* arg) {
   emscripten_proxy_finish(ctx);
 }
 
-int emscripten_proxy_sync(em_proxying_queue* q,
+bool emscripten_proxy_sync(em_proxying_queue* q,
                           pthread_t target_thread,
                           void (*func)(void*),
                           void* arg) {
@@ -422,32 +422,32 @@ int emscripten_proxy_sync(em_proxying_queue* q,
     q, target_thread, call_then_finish_task, &t);
 }
 
-static int do_proxy_callback(em_proxying_queue* q,
-                             pthread_t target_thread,
-                             void (*func)(em_proxying_ctx* ctx, void*),
-                             void (*callback)(void*),
-                             void (*cancel)(void*),
-                             void* arg,
-                             em_proxying_ctx* ctx) {
+static bool do_proxy_callback(em_proxying_queue* q,
+                              pthread_t target_thread,
+                              void (*func)(em_proxying_ctx* ctx, void*),
+                              void (*callback)(void*),
+                              void (*cancel)(void*),
+                              void* arg,
+                              em_proxying_ctx* ctx) {
   em_proxying_ctx_init_callback(
     ctx, q, pthread_self(), func, callback, cancel, arg);
   if (!do_proxy(q, target_thread, (task){call_with_ctx, cancel_ctx, ctx})) {
     free_ctx(ctx);
-    return 0;
+    return false;
   }
-  return 1;
+  return true;
 }
 
-int emscripten_proxy_callback_with_ctx(em_proxying_queue* q,
-                                       pthread_t target_thread,
-                                       void (*func)(em_proxying_ctx* ctx,
-                                                    void*),
-                                       void (*callback)(void*),
-                                       void (*cancel)(void*),
-                                       void* arg) {
+bool emscripten_proxy_callback_with_ctx(em_proxying_queue* q,
+                                        pthread_t target_thread,
+                                        void (*func)(em_proxying_ctx* ctx,
+                                                     void*),
+                                        void (*callback)(void*),
+                                        void (*cancel)(void*),
+                                        void* arg) {
   em_proxying_ctx* ctx = malloc(sizeof(*ctx));
   if (ctx == NULL) {
-    return 0;
+    return false;
   }
   return do_proxy_callback(q, target_thread, func, callback, cancel, arg, ctx);
 }
@@ -477,12 +477,12 @@ static void callback_cancel(void* arg) {
   }
 }
 
-int emscripten_proxy_callback(em_proxying_queue* q,
-                              pthread_t target_thread,
-                              void (*func)(void*),
-                              void (*callback)(void*),
-                              void (*cancel)(void*),
-                              void* arg) {
+bool emscripten_proxy_callback(em_proxying_queue* q,
+                               pthread_t target_thread,
+                               void (*func)(void*),
+                               void (*callback)(void*),
+                               void (*cancel)(void*),
+                               void* arg) {
   // Allocate the em_proxying_ctx and the user ctx as a single block that will
   // be freed when the `em_proxying_ctx` is freed.
   struct block {
@@ -491,7 +491,7 @@ int emscripten_proxy_callback(em_proxying_queue* q,
   };
   struct block* block = malloc(sizeof(*block));
   if (block == NULL) {
-    return 0;
+    return false;
   }
   block->cb_ctx = (callback_ctx){func, callback, cancel, arg};
   return do_proxy_callback(q,
@@ -612,18 +612,51 @@ typedef struct proxied_js_func_t {
 static void run_js_func(void* arg) {
   proxied_js_func_t* f = (proxied_js_func_t*)arg;
   f->result = _emscripten_receive_on_main_thread_js(
-    f->funcIndex, f->emAsmAddr, f->callingThread, f->bufSize, f->argBuffer);
+    f->funcIndex, f->emAsmAddr, f->callingThread, f->bufSize, f->argBuffer, 0, 0);
   if (f->owned) {
     free(f->argBuffer);
     free(f);
   }
 }
 
+static void run_js_func_with_ctx(em_proxying_ctx* ctx, void* arg) {
+  proxied_js_func_t* f = (proxied_js_func_t*)arg;
+  _emscripten_receive_on_main_thread_js(
+    f->funcIndex, f->emAsmAddr, f->callingThread, f->bufSize, f->argBuffer, ctx, arg);
+
+  // run_js_func_with_ctx is always synchronously proxied and therefore arg
+  // should never be owned on the main thread (i.e. the argument here always
+  // exists on the stack of the calling thread, it's never copied/malloced).
+  assert(!f->owned);
+}
+
+void _emscripten_run_js_on_main_thread_done(void* ctx, void* arg, double result) {
+  proxied_js_func_t* f = (proxied_js_func_t*)arg;
+  f->result = result;
+  emscripten_proxy_finish(ctx);
+}
+
+/*
+ * The 'proxy_mode' argument to _emscripten_run_js_on_main_thread has 3 possible
+ * values:
+ *
+ * - PROXY_ASYNC: Returns immediately on the calling thread, does not signal
+ * - PROXY_SYNC: Synchronous on the calling thread, and also on the main thread
+ * - PROXY_SYNC_ASYNC: Synchronous on the calling thread, but async on the main
+ *   thread.
+ *
+ * Note: 'PROXY_SYNC_ASYNC' is only passed when a function is marked as
+ * both "__async" and "__proxy: 'sync'"
+ */
+#define PROXY_ASYNC 0
+#define PROXY_SYNC 1
+#define PROXY_SYNC_ASYNC 2
+
 double _emscripten_run_js_on_main_thread(int func_index,
                                          void* em_asm_addr,
                                          int buf_size,
                                          double* buffer,
-                                         bool sync) {
+                                         int proxyMode) {
   proxied_js_func_t f = {
     .funcIndex = func_index,
     .emAsmAddr = em_asm_addr,
@@ -636,9 +669,15 @@ double _emscripten_run_js_on_main_thread(int func_index,
   em_proxying_queue* q = emscripten_proxy_get_system_queue();
   pthread_t target = emscripten_main_runtime_thread_id();
 
-  if (sync) {
-    if (!emscripten_proxy_sync(q, target, run_js_func, &f)) {
-      assert(false && "emscripten_proxy_sync failed");
+  if (proxyMode != PROXY_ASYNC) {
+    int rtn;
+    if (proxyMode == PROXY_SYNC_ASYNC) {
+      rtn = emscripten_proxy_sync_with_ctx(q, target, run_js_func_with_ctx, &f);
+    } else {
+      rtn = emscripten_proxy_sync(q, target, run_js_func, &f);
+    }
+    if (!rtn) {
+      assert(false && "emscripten_proxy_sync_with_ctx failed");
       return 0;
     }
     return f.result;
