@@ -16,9 +16,15 @@
 #include <string.h>
 #include <sys/time.h>
 
+#define TIMEOUT_MS 300
+
+void sleep_ms(int ms) {
+  usleep(ms * 1000);
+}
+
 int64_t timeval_delta_ms(struct timeval* begin, struct timeval* end) {
   int64_t delta_s = end->tv_sec - begin->tv_sec;
-  int64_t delta_us =  end->tv_usec -  begin->tv_usec;
+  int64_t delta_us = end->tv_usec - begin->tv_usec;
   assert(delta_s >= 0);
   return (delta_s * 1000) + (delta_us / 1000);
 }
@@ -29,12 +35,12 @@ void test_timeout_without_fds() {
   struct timeval begin, end;
 
   gettimeofday(&begin, NULL);
-  assert(poll(NULL, 0, 1000) == 0);
+  assert(poll(NULL, 0, TIMEOUT_MS) == 0);
   gettimeofday(&end, NULL);
 
   int64_t duration = timeval_delta_ms(&begin, &end);
   printf(" -> duration: %lld ms\n", duration);
-  assert(duration >= 1000);
+  assert(duration >= TIMEOUT_MS);
 }
 
 // Check if timeout works with fds without events
@@ -47,22 +53,22 @@ void test_timeout_with_fds_without_events() {
 
   gettimeofday(&begin, NULL);
   struct pollfd fds = {pipe_a[0], 0, 0};
-  assert(poll(&fds, 1, 1000) == 0);
+  assert(poll(&fds, 1, TIMEOUT_MS) == 0);
   gettimeofday(&end, NULL);
 
   int64_t duration = timeval_delta_ms(&begin, &end);
   printf(" -> duration: %lld ms\n", duration);
-  assert(duration >= 1000);
+  assert(duration >= TIMEOUT_MS);
 
   close(pipe_a[0]); close(pipe_a[1]);
 }
 
 int pipe_shared[2];
 
-void *write_after_2s(void * arg) {
+void *write_after_sleep(void * arg) {
   const char *t = "test\n";
 
-  sleep(2);
+  sleep_ms(TIMEOUT_MS);
   write(pipe_shared[1], t, strlen(t));
 
   return NULL;
@@ -82,15 +88,15 @@ void test_unblock_poll() {
     {pipe_a[0], POLLIN, 0},
     {pipe_shared[0], POLLIN, 0},
   };
-  assert(pthread_create(&tid, NULL, write_after_2s, NULL) == 0);
   gettimeofday(&begin, NULL);
+  assert(pthread_create(&tid, NULL, write_after_sleep, NULL) == 0);
   assert(poll(fds, 2, -1) == 1);
   gettimeofday(&end, NULL);
   assert(fds[1].revents & POLLIN);
 
   int64_t duration = timeval_delta_ms(&begin, &end);
   printf(" -> duration: %lld ms\n", duration);
-  assert(duration >= 1000);
+  assert(duration >= TIMEOUT_MS);
 
   pthread_join(tid, NULL);
 
@@ -105,11 +111,11 @@ void *do_poll_in_thread(void * arg) {
   struct pollfd fds = {pipe_shared[0], POLLIN, 0};
   assert(poll(&fds, 1, 4000) == 1);
   gettimeofday(&end, NULL);
-  assert(fds.events & POLLIN);
+  assert(fds.revents & POLLIN);
 
   int64_t duration = timeval_delta_ms(&begin, &end);
   printf(" -> duration: %lld ms\n", duration);
-  assert((duration >= 1000) && (duration < 4000));
+  assert((duration >= TIMEOUT_MS) && (duration < 4000));
 
   return NULL;
 }
@@ -125,7 +131,7 @@ void test_poll_in_threads() {
   assert(pthread_create(&tid1, NULL, do_poll_in_thread, NULL) == 0);
   assert(pthread_create(&tid2, NULL, do_poll_in_thread, NULL) == 0);
 
-  sleep(2);
+  sleep_ms(2 * TIMEOUT_MS);
   write(pipe_shared[1], t, strlen(t));
 
   pthread_join(tid1, NULL);
