@@ -181,7 +181,7 @@ def apply_static_code_hooks(forwarded_json, code):
 def generate_js_compiler_input_hash(symbols_only=False):
   # We define a cache hit as when all the settings and all the `--js-library`
   # contents are identical.
-  # Ignore certain settings that can are no relevant to library deps.  Here we
+  # Ignore certain settings that are not relevant to library deps.  Here we
   # skip PRE_JS_FILES/POST_JS_FILES which don't affect the library symbol list
   # and can contain full paths to temporary files.
   skip_settings = {'PRE_JS_FILES', 'POST_JS_FILES'}
@@ -343,7 +343,7 @@ def compile_javascript_cached():
   # Avoiding using the cache when generating struct info since
   # this step is performed while the cache is locked.
   # Sadly we have to skip the caching whenever we have user JS libraries.  This is because
-  # these libraries can import arbirary other JS files (either vis node's `import` or via #include)
+  # these libraries can import arbitrary other JS files (either vis node's `import` or via #include)
   if DEBUG or settings.BOOTSTRAPPING_STRUCT_INFO or config.FROZEN_CACHE or settings.JS_LIBRARIES:
     return compile_javascript()
 
@@ -382,20 +382,22 @@ def emscript(in_wasm, out_wasm, outfile_js, js_syms, finalize=True, base_metadat
 
   if not settings.WASM_BIGINT and metadata.em_js_funcs:
     for em_js_func, raw in metadata.em_js_funcs.items():
-      c_sig = raw.split('<::>')[0].strip('()')
-      if not c_sig or c_sig == 'void':
-        c_sig = []
+      args, _ = raw.split('<::>', 1)
+      args = args[1:-1].strip()
+
+      if not args or args == 'void':
+        args = []
       else:
-        c_sig = c_sig.split(',')
+        args = args.split(',')
       signature = metadata.em_js_func_types.get(em_js_func)
-      if signature and len(signature.params) != len(c_sig):
-        diagnostics.warning('em-js-i64', 'using 64-bit arguments in EM_JS function without WASM_BIGINT is not yet fully supported: `%s` (%s, %s)', em_js_func, c_sig, signature.params)
+      if signature and len(signature.params) != len(args):
+        diagnostics.warning('em-js-i64', 'using 64-bit arguments in EM_JS function without WASM_BIGINT is not yet fully supported: `%s` (%s, %s)', em_js_func, args, signature.params)
 
   asm_consts = create_asm_consts(metadata)
   em_js_funcs = create_em_js(metadata)
 
   if settings.SIDE_MODULE:
-    # When building side modules, valid the EM_ASM and EM_JS string by running
+    # When building side modules, validate the EM_ASM and EM_JS string by running
     # them through node.  Without this step, syntax errors are not surfaced
     # until runtime.
     # We use subprocess directly here rather than shared.check_call since
@@ -588,16 +590,20 @@ def finalize_wasm(infile, outfile, js_syms):
 
   # For sections we no longer need, strip now to speed subsequent passes.
   # If Binaryen is not needed, this is also our last chance to strip.
-  strip_sections = []
+  # The `llvm.func_attr.annotate` are created by `[[clang::annotate]]` or
+  # similar.  We have some plans to use them for emscripten metadata but
+  # they should not persist in the final binary.
+  strip_sections = ['llvm.func_attr.annotate.*']
   if not settings.EMIT_PRODUCERS_SECTION:
     strip_sections += ['producers']
   if not need_name_section:
     strip_sections += ['name']
+  if not settings.GENERATE_DWARF:
+    strip_sections += ['.debug*']
 
-  if strip_sections or not settings.GENERATE_DWARF:
+  if strip_sections:
     building.save_intermediate(outfile, 'strip.wasm')
-    building.strip(infile, outfile, debug=not settings.GENERATE_DWARF,
-                   sections=strip_sections)
+    building.strip_sections(infile, outfile, strip_sections)
 
   metadata = get_metadata(outfile, outfile, modify_wasm, args)
 
@@ -789,8 +795,8 @@ def create_em_js(metadata):
   for name, raw in metadata.em_js_funcs.items():
     assert separator in raw
     args, body = raw.split(separator, 1)
-    args = args[1:-1]
-    if args == 'void':
+    args = args[1:-1].strip()
+    if not args or args == 'void':
       args = []
     else:
       args = args.split(',')
@@ -956,10 +962,10 @@ def install_debug_wrapper(sym):
   # The emscripten stack functions are called very early (by writeStackCookie) before
   # the runtime is initialized so we can't create these wrappers that check for
   # runtimeInitialized.
-  if sym.startswith(('__asan_', 'emscripten_stack_', '_emscripten_stack_')):
+  if sym.startswith(('__asan_', '__lsan_', 'emscripten_stack_', '_emscripten_stack_')):
     return False
   # `__trap` can occur before the runtime is initialized since it is used in abort.
-  # `emscripten_get_sbrk_ptr` can be called prior to runtime initialzaion by
+  # `emscripten_get_sbrk_ptr` can be called prior to runtime initialization by
   # the dynamic linking code.
   return sym not in ['__trap', 'emscripten_get_sbrk_ptr']
 
@@ -1068,7 +1074,7 @@ def create_receiving(function_exports, other_exports, library_symbols, aliases):
       else:
         assignment = f"Module['{mangled}']"
     elif not js_manipulation.isidentifier(mangled):
-      # Symbol is not a valid JS identify and also not exported. In this case we
+      # Symbol is not a valid JS identifier and also not exported. In this case we
       # have nothing to do here.
       continue
 
@@ -1196,12 +1202,14 @@ def create_pointer_conversion_wrappers(metadata):
     '_wasmfs_node_record_dirent': '_pp_',
     '__dl_seterr': '_pp',
     '_emscripten_run_js_on_main_thread': '__p_p_',
+    '_emscripten_run_js_on_main_thread_done': '_pp_',
     '_emscripten_thread_exit': '_p',
     '_emscripten_thread_init': '_p_____',
     '_emscripten_thread_free_data': '_p',
     '_emscripten_dlsync_self_async': '_p',
     '_emscripten_proxy_dlsync_async': '_pp',
     '_emscripten_wasm_worker_initialize': '_p_',
+    '_emscripten_proxy_poll_finish': '_pp_',
     '_wasmfs_rename': '_pp',
     '_wasmfs_readlink': '_pp',
     '_wasmfs_truncate': '_p_',
@@ -1257,7 +1265,7 @@ function applySignatureConversions(wasmExports) {
     # dynCall_ functions are generated by binaryen.  They all take a
     # function pointer as their first argument.
     # The second part of this check can be removed once the table64
-    # llvm changs lands.
+    # llvm changes land.
     if symbol.startswith('dynCall_') and functype.params[0] == webassembly.Type.I64:
       sig = symbol.split('_')[-1]
       sig = ['p' if t == 'p' else '_' for t in sig]
